#ifndef __I2C_HPP__
#define __I2C_HPP__

//	   TWBR	(TWPS0  TWPS1  0)
#define	I2C_BAUD_0099	66	// 14.7456 / (16 + 2 * 66 * 4^0) = 0,0996 MHz
#define	I2C_BAUD_0388	11	// 14.7456 / (16 + 2 * 11 * 4^0) = 0,388 MHz
#define	I2C_BAUD_0600	4	// 14.7456 / (16 + 2 * 4 * 4^0) = 0,600 MHz
#define	I2C_BAUD_0737  2	// 14.7456 / (16 + 2 * 2 * 4^0) = 0,737 MHz

#define	I2C_STNDBY			(!(TWCR & (1 << TWEN)))
////#define	I2C_STNDBY			((TWCR & (1 << TWSTO)))
#define	I2C_MODERD			0x01
#define	I2C_MAXPRTY			1

#define	I2C_START			0x00
#define	I2C_RESTART			0x01
#define	I2C_HSEND			0x02
#define	I2C_DSEND			0x03
#define	I2C_STOP				0x04
#define	I2C_BREAK			0x05

class I2CQuery	{
	private:
	public:
	volatile	class I2CQuery	*next;
							bool	ready;
	volatile	unsigned	char	error;
	volatile	unsigned	char	prty;
	volatile	unsigned	char	baud;

	volatile	unsigned	char	numb;				//	   
	volatile	unsigned	char	asize;			//	     
	volatile	unsigned	int	addr;				//	  
	volatile	unsigned	char	dsize,*data;
  //--------------------------------------------------------
	I2CQuery(void)							
		:	numb(0), prty(0), baud(I2C_BAUD_0099), ready(false), error(0xFF), next(NULL),
			asize(0), addr(NULL), dsize(0), data(NULL)	{
			}
  //--------------------------------------------------------
	I2CQuery(unsigned	char	_dev)	
		:	prty(0), baud(I2C_BAUD_0099), ready(false), error(0xFF), next(NULL),
			asize(0), addr(NULL), dsize(0), data(NULL)	{
			numb	=	_dev;
			}
  //--------------------------------------------------------
	I2CQuery(unsigned	char	_dev,unsigned	char	_prty)	
		:	baud(I2C_BAUD_0099), ready(false), error(0xFF), next(NULL),
		 	asize(0), addr(NULL), dsize(0), data(NULL)	{
			numb	=	_dev;
			prty	=	_prty % (unsigned	char)(I2C_MAXPRTY + 1);
		 	};
  //--------------------------------------------------------
	I2CQuery(unsigned	char	_dev,unsigned	char	_prty,unsigned	char	_baud)	
		:	ready(false), error(0xFF), next(NULL), 
			asize(0), addr(NULL), dsize(0), data(NULL)	{
			numb	=	_dev;
			prty	=	_prty % (unsigned	char)(I2C_MAXPRTY + 1); 
			baud	=	_baud; 
			}
  //--------------------------------------------------------
	unsigned	char	isReady(void)	{
		unsigned	char	ecode;
		if(!ready)	return	0x00;
		ecode	=	error;
/////////////////////////////////////////////		delete	this;
		return	ecode;
		}
	};

class ClassI2C	{
  	private:
  	volatile	unsigned	char	cntr,*buffer;
  	volatile	unsigned	char	state;
  	//--------------------------------------------------------
  	inline	void  setbaud(unsigned	char	_baud)	{	TWBR	=	_baud;	}
  	//--------------------------------------------------------
  	public:
  	volatile	I2CQuery			*EndOfList[I2C_MAXPRTY + 1];
  	volatile	I2CQuery			*cpoint[I2C_MAXPRTY + 1];
  	volatile	I2CQuery			*wpoint;
  	//--------------------------------------------------------
  	ClassI2C(void)	{
  		TWCR	=	(1 << TWIE) | (1 << TWSTO);
		for(char	i=0;i<(I2C_MAXPRTY + 1);i++)	{
	  		EndOfList[i] 			=	NULL;
			cpoint[i] 				=	NULL;
			EndOfList[i]->next	=	NULL;
			cpoint[i]->next		=	NULL;
			}
		wpoint					=	NULL;                               
  		}
  	//--------------------------------------------------------
	void	nextQuery()	{
  		for(char	i=0;i<(I2C_MAXPRTY + 1);i++)	{
  			wpoint	=	cpoint[i];
  			if(NULL	!=	wpoint)	{
  				setbaud(wpoint->baud);
  				state			=	I2C_START;
  				TWCR	|=	(1 << TWINT) | (1 << TWSTA) | (1 << TWEN) & ~(1 << TWSTO);
  				return;
  				}
  			}
////		TWCR	&= ~((1 << TWEN) | (1 << TWSTO));
		TWCR	&= ~(1 << TWEN);
		return;
  		}
  	//--------------------------------------------------------
  	void	AddQuery(I2CQuery *query)	{
		unsigned	char		sreg;
  		unsigned	char		prty;

		query->ready	=	false;
  		prty	=	query->prty	%=	(I2C_MAXPRTY + 1);
		sreg	=	SREG;
		__disable_interrupt();
		EndOfList[prty]->next	=	query;
		EndOfList[prty]			=	EndOfList[prty]->next;
		EndOfList[prty]->next	=	NULL;
 		//	  ?
  		if(NULL	==	cpoint[prty])	cpoint[prty]	=	EndOfList[prty];
		//	 ?
  		if(I2C_STNDBY)	nextQuery();
		SREG	=	sreg;
  		return;
  		}
  	//--------------------------------------------------------
  	void	interruptHandle(void)	{
  		unsigned	char	byte;
  		unsigned	char	twcr;

//	    !!!
//  		unsigned	char	twsr;
//		twsr	=	TWSR	& 0xF8;

		//	!!!!! STA  STO   !!!!!
		twcr	=	TWCR & ~((1 << TWSTA) | (1 << TWSTO));
		switch(state)	{
			///////////////////////////////////////
			case	I2C_START:	case	I2C_RESTART:
				if(I2C_START == state)	{	//	     -  
	  				state		=	I2C_HSEND;
					cntr		=	wpoint->asize;
	  				buffer	=	(unsigned	char	*)&wpoint->addr;
					if(2 == cntr)	buffer++;
					byte		=	wpoint->numb	&	~I2C_MODERD;	//	     
					}
				else	{							//	     -  
	  				state		=	I2C_DSEND;
					cntr		=	wpoint->dsize;
	  				buffer	=	wpoint->data;
					byte		=	wpoint->numb	|	I2C_MODERD;		//	    
					}
  				TWDR	=	byte;
				TWCR	=	twcr;
	  			break;
			///////////////////////////////////////
  			case	I2C_HSEND:
		  		if(cntr-- > 0)	{
		  			TWDR	=	*buffer--;
					TWCR	=	twcr;
		  			}
		  		else	{
	  				if(wpoint->numb & I2C_MODERD)	{	//	   
  						state	=	I2C_RESTART;
  						TWCR	=	twcr	|	(1 << TWSTA);
  						}
	  				else	{									//	   
  						state    =	I2C_DSEND;
						cntr		=	wpoint->dsize - 1;
						buffer	=	wpoint->data;
		  				TWDR		=	*buffer++;
						TWCR		=	twcr;
						}
					}
	  			break;
			case	I2C_DSEND:
		  		if(cntr > 0)	{
		  			if(wpoint->numb & I2C_MODERD)  {
		  				//	    nextQuery
		  				twcr	|=	(1 << TWEA);
		  				if(cntr	<	wpoint->dsize)	*buffer++	=	TWDR;
		  				if(1 == cntr)					twcr	&=	~(1 << TWEA);
		  				}
		  			else					  				TWDR	=	*buffer++;
		  			cntr--;
					TWCR	=	twcr;
					break;
		  			}
	  			if(wpoint->numb & I2C_MODERD) *buffer	=	TWDR;

  				TWCR	=	twcr | (1 << TWSTO);

				wpoint->ready	=	true;
				cpoint[wpoint->prty]	=	cpoint[wpoint->prty]->next;
				nextQuery();
				break;
//			///////////////////////////////////////
//	  		case	I2C_STOP:
//		  		debcntr3++;
//				wpoint->ready	=	true;
//				cpoint[wpoint->prty]	=	cpoint[wpoint->prty]->next;
//	  			nextQuery();
//	  			break;
			}
		return;
		}
  	};
extern	ClassI2C	*pI2C;
#endif  //  __I2C_HPP__
