#ifndef __SMH2010_HPP_
#define __SMH2010_HPP_

typedef				bool	BOOL;
typedef unsigned  char	BYTE;
typedef unsigned  int   WORD;
typedef unsigned  long  DWORD;
typedef 				int   INT;
typedef 				long  LONG;
typedef           float FLOAT;

typedef           bool  *pBOOL;
typedef unsigned  char  *pBYTE;
typedef unsigned  int   *pWORD;
typedef unsigned  long 	*pDWORD;
typedef 				int   *pINT;
typedef				long 	*pLONG;
typedef           float *pFLOAT;

#include  <ina90.h>  
#include  <iom128.h>
#include  <stdlib.h>
#include  <pgmspace.h>

#define   DISP_Text20x4
#include  <display.hpp>

#include	<I2c.hpp>
#include	<dac.hpp>
#include	<Debugger.hpp>


typedef struct _real_time_ {
  volatile  char            seconds;
  volatile  char            minutes;
  volatile  char            hours;
  volatile  unsigned  long  days;
  } sTime;

#define IODATA_PORT PORTB
#define IODATA_DIR  DDRB
#define IODATA_PIN  PINB

#define CTRL1_PORT  PORTE
#define CTRL1_DIR   DDRE
#define CTRL1_PIN   PINE

#define CTRL2_PORT  PORTD
#define CTRL2_DIR   DDRD
#define CTRL2_PIN   PIND                

#define MASK_TXEN0  0x04
#define MASK_TXEN1  0x08
#define MASK_RKWR   0x10
#define MASK_RKRD   0x20
#define MASK_PFO    0x40                                                                             

#ifdef	__SMH2010_C__
#define MASK_LIGHT	0x80
#endif	//	__SMH2010_C__

#define MASK_DRS    0x10
#define MASK_DEN    0x20
#define MASK_KBDRD  0x40
#define MASK_TSTRD  0x80

#define	WDTTO_0016	0x00
#define	WDTTO_0032	0x01
#define	WDTTO_0065	0x02
#define	WDTTO_0130	0x03
#define	WDTTO_0260	0x04
#define	WDTTO_0520	0x05
#define	WDTTO_1000	0x06
#define	WDTTO_2100	0x07

#define SetBitsPort(port,mask)      port |= mask
#define ResetBitsPort(port,mask)    port &= ~mask
#define ToggleBitsPort(port,mask)   port ^= mask
#define WritePort(port,byte)        port = byte
#define ResetBits(port,mask)        ResetBitsPort(port,mask)
#define TwoClckWait()               asm(" NOP");	asm(" NOP")
                                    
//        14.7456 MHz ( 4,3402(7) mkS)
#define TIMER3_PRESC 	0x03     	//  Prescale  = 64
#define TIMER3_FltTICK 	4.34027    	//	mkS

//      14.7456 MHz (998.236(8) mkS)
#define SYS_TIMER_CONST 230
#define SYS_TIMER_PRESC 0x03  //  Prescale  = 64

//  Real Time Clock ( 1S )
#define RTC_CONST 32
#define RTC_PRESC 0x07        //  Prescale = 1024

#define	EXTRAM_MAXSIZE		(32U * 1024U)

/******************************************************************/
/*    /       */
/*   DEN  DRS               */
/*       - 1               */
/******************************************************************/
extern	volatile  		__no_init sTime Time;
class ClassBiosSMH  {
  private:
						unsigned	char	 rsave[16];			 //       
    volatile      unsigned  int   input_rk[3];      //      
    volatile      unsigned  long  scan_kbd[3];      //    
    volatile      unsigned  char  test_rk;          //        
    volatile      char            cntr_irk;         //     
    volatile      char            cntr_kbd;         //    
//    volatile  		__no_init sTime Time;
    //---------------------------------------------------------------------------------
    void  (*__call_0000)(void);                        //  ??????? ?????????? RESET ??????????
  //---------------------------------------------------------------------------------
  public:
    volatile      int            m_lSystemTicks;   //    
    volatile      unsigned	char	DOutputs;         //      
    volatile      unsigned	int   WDTcntr;
#ifdef	__DEBUGGER__
	volatile			unsigned	long	KbdScanCode_DEB;
	volatile			unsigned	int	DInputs_DEB;
   volatile      	unsigned	char	DOutputs_DEB;
#endif	//	__DEBUGGER__
	
    //---------------------------------------------------------------------------------
    inline  void  _reset(void)	{
    	__disable_interrupt();
    	__call_0000();
    	}
    //-------------------------------------------------------------------
    ClassBiosSMH(void)  : 	cntr_kbd(0),
                          	cntr_irk(0),
                          	m_lSystemTicks(0),
                          	DOutputs(0),
                         	WDTcntr(0)
#ifdef	__DEBUGGER__
									,KbdScanCode_DEB(0L)
									,DInputs_DEB(0)
   								,DOutputs_DEB(0)
#endif	//	__DEBUGGER__
                          {

		SFIOR	|=	(1 << PUD);
#ifdef	__SMH2010_C__
		SetBitsPort(CTRL1_DIR,MASK_LIGHT);	//	   
#endif	//	__SMH2010_C__

      SetBitsPort(CTRL1_DIR,(MASK_RKWR | MASK_RKRD | MASK_TXEN0 | MASK_TXEN1));
      SetBitsPort(CTRL1_PORT,(MASK_RKRD));
      SetBitsPort(CTRL2_DIR,(MASK_DRS | MASK_DEN | MASK_KBDRD | MASK_TSTRD));
      SetBitsPort(CTRL2_PORT,(MASK_DRS | MASK_DEN | MASK_KBDRD | MASK_TSTRD));
		WritePort(IODATA_DIR,0x00); 	//     

//	  low_level_init
//////      MCUCR	|=	(1 << SRE);	//  XRAM enabled
      
      ResetTxEnable(0);
      ResetTxEnable(1);
      AtomWriteORK();

      //  Set up pointer for function to reset execution
      __call_0000 = 0x0000;
      //  EEPROM
      EECR  |=  0x01; //  Read enable
      };
    //---------------------------------------------------------------------------------
    void  Init(void)  {
        //     
        EIFR  = 0xFF;
        //  Real Time Clock Timer0  8 bit
        ASSR  |=  (1  <<  AS0);	
        TCCR0 |=  RTC_PRESC;		
        TCCR0 |=  (1 << WGM01);
        OCR0  =   RTC_CONST - 1;	
        TIMSK |=  (1 << OCIE0);
        Time.seconds %= 60;	Time.minutes %= 60;	Time.hours %= 24;
        //     Timer2  8 bit
        TCNT2	=	0x00;				
        OCR2	=	SYS_TIMER_CONST;
        TIFR	|=  (1 << OCF2);	
        TIMSK 	|=  (1 << OCIE2);

        //     Timer3  16 bit
        //    ,    
        //      (   clck) 
        TCNT3	=	0;

        scan_kbd[0] = scan_kbd[1] = scan_kbd[2] = 0xFFFFFFFFL;
        
//	  low_level_init
//        EnableXRAM();

  		  _init_display();
        
        //  START -    Timer2  8 bit
        TCCR2 |=  ((1 << WGM21) | SYS_TIMER_PRESC);
        //  START -    Timer3  16 bit
        TCCR3B |=  TIMER3_PRESC;  
        //  Sleep mode = Power-save Sleep enable
        MCUCR |=  ((1 << SM1) | (1 << SM0) | (1 << SE));

#ifdef	__SMH2010_C__
		  //  Low level
        //	(   )
        EIMSK |=  (1 << INT6);
		  LightON();
#else
#ifdef	__BATTERY__
		  //  Low level
        EIMSK |=  (1 << INT6);
#endif
#endif
		  __enable_interrupt();
        	return;
      }
    //-------------------------------------------------------------------
    inline	void  _Sleep(void)  {
    	unsigned	char	*bpoint;

      _savescr();
      ResetWDT();
      DisableWDT();

      //  	      
      bpoint		=	rsave;
    	*bpoint++	=	DDRA;    	*bpoint++	=	DDRB;
    	*bpoint++	=	DDRC;    	*bpoint++	=	DDRD;
    	*bpoint++	=	DDRE;    	*bpoint++	=	DDRF;
      //     
    	*bpoint++	=	EIMSK;    	*bpoint++	=	TIMSK;
    	*bpoint++	=	ETIMSK;    	*bpoint++	=	SPCR;
    	*bpoint++	=	TWCR;      	*bpoint++	=	ACSR;
    	*bpoint++	=	ADCSRA;
    	*bpoint++	=	UCSR0B;
    	*bpoint++	=	UCSR1B;
    	*bpoint++	=	ADMUX;
      //		        
      UCSR0B	=	UCSR1B	=	EIMSK	=	TIMSK	=	ETIMSK	=	SPCR	=	TWCR	=	ACSR	=	ADCSRA	=	0x00;
    	DDRA  = DDRB  = DDRC  = DDRD  = DDRE  = DDRF  = 0x00;
    	//	  
    	ADMUX	=	0xC0;

      //    RTC
      TIMSK |=  (1 << OCIE0); 	//  Timer0
    	}
    //-------------------------------------------------------------------
    inline	void  _WakeUp(void)  {
    	unsigned	char	*bpoint;

		#ifdef	__DEBUGGER__
		KbdScanCode_DEB	=	0L;
		DInputs_DEB			=	0;
		DOutputs_DEB		=	0;
		#endif	//	__DEBUGGER__

      //      
      bpoint	=	rsave;
    	DDRA	=	*bpoint++;    	DDRB	=	*bpoint++;
    	DDRC	=	*bpoint++;    	DDRD	=	*bpoint++;
    	DDRE	=	*bpoint++;    	DDRF	=	*bpoint++;
      //     
    	EIMSK		=	*bpoint++;    	TIMSK	=	*bpoint++;
    	ETIMSK	=	*bpoint++;    	SPCR	=	*bpoint++;
    	TWCR		=	*bpoint++;     ACSR	=	*bpoint++;
    	ADCSRA	=	*bpoint++;
    	UCSR0B	=	*bpoint++;
    	UCSR1B	=	*bpoint++;
    	ADMUX		=	*bpoint++;

      AtomWriteORK();

      _init_display();
      _restorescr();
      
      //	 WDT
		EnableWDT(WDTTO_2100);
      ResetWDT();
    	}
    //-------------------------------------------------------------------
    inline	void  Sleep(void)  {
      char  h,m,s;
      long  time,oldtime;

		_Sleep();
      GetRealTime(&h,&m,&s);
      oldtime = ((long)h * 3600L) + ((long)m * 60L) + (long)s;
      DisableXRAM();

      __enable_interrupt();
      do  __sleep();	
      	while(0 == (PINE  & (1 << PINE6)));
      __disable_interrupt();
      EnableXRAM();

      //       sleep 
      GetRealTime(&h,&m,&s);
      time = ((long)h * 3600L) + ((long)m * 60L) + (long)s;
      //      EEPROM
      if(2L < (time - oldtime))	_reset();

		_WakeUp();
      __enable_interrupt();
      }
    //-------------------------------------------------------------------
    inline  void  AtomReadORK(void)  {
    unsigned  char  sreg;

      sreg  = SREG; __disable_interrupt();
      ResetBitsPort(CTRL2_PORT,MASK_TSTRD);
      TwoClckWait();
      test_rk = IODATA_PIN;
      SetBitsPort(CTRL2_PORT,MASK_TSTRD);
      SREG  = sreg;
      return;
      }
    //-------------------------------------------------------------------
    inline  void  AtomWriteORK(void)  {
    unsigned  char  sreg;

      sreg  = SREG; __disable_interrupt();
      WritePort(IODATA_DIR,0xFF);           //    
      WritePort(IODATA_PORT,DOutputs);      //    
      SetBitsPort(CTRL1_PORT,MASK_RKWR);
      ResetBitsPort(CTRL1_PORT,MASK_RKWR);
      WritePort(IODATA_DIR,0x00);           //    
      SREG  = sreg;
      return;
      }
    //-------------------------------------------------------------------
    inline  void  AtomReadIRK(void)  {
      unsigned  char  sreg;

      input_rk[cntr_irk]  = 0;
      //     
      //  , ,    
      //     ?????
      sreg  = SREG; __disable_interrupt();
      ResetBitsPort(CTRL1_PORT,MASK_RKRD);                 //   RKRD - 0
      TwoClckWait();
      input_rk[cntr_irk] = (unsigned  int)IODATA_PIN;      //   
      SetBitsPort(CTRL1_PORT,MASK_RKRD);                   //   RKRD - 1
      SREG  = sreg;

      sreg  = SREG; __disable_interrupt();
      ResetBitsPort(CTRL2_PORT,MASK_KBDRD);                         //   KBDRD - 0
      TwoClckWait();
      input_rk[cntr_irk]  |= ((unsigned int)(IODATA_PIN & 0xF0) << 4);  //   
      SetBitsPort(CTRL2_PORT,MASK_KBDRD);                           //   KBDRD - 1
      SREG  = sreg;
      cntr_irk++; cntr_irk %= 3;
      return;
      }
    //-------------------------------------------------------------------
    inline  void  AtomReadKBD(void)  {
      unsigned  char  mask,sreg;
                char  i;
      
		cntr_kbd++; cntr_kbd %= 3;
   	mask  = 0x80;
	   scan_kbd[cntr_kbd] = 0L;
   	//   
	   for(i=0;i<8;i++)  {
   	  scan_kbd[cntr_kbd] <<=  4;
	     sreg  = SREG; __disable_interrupt();
   	  WritePort(IODATA_DIR,0xFF);                 //     
	     WritePort(IODATA_PORT,~mask);               //   0   
   	  ResetBitsPort(CTRL2_PORT,MASK_DRS);         //      
	     WritePort(IODATA_DIR,0x00);                 //     
   	  SREG  = sreg;
	
   	  sreg  = SREG; __disable_interrupt();
	     ResetBitsPort(CTRL2_PORT,MASK_KBDRD);       //    
   	  scan_kbd[cntr_kbd] |= IODATA_PIN & 0x0F;    //   
	     SetBitsPort(CTRL2_PORT,MASK_KBDRD);         //    
   	  SetBitsPort(CTRL2_PORT,MASK_DRS);           //     
	     SREG  = sreg;
   	  mask  >>= 1;
	     }
      return;
      }
    //-------------------------------------------------------------------
    inline  void  DisplayCtrlWrite(unsigned char  byte)  {
      unsigned  char  sreg;
      
        sreg  = SREG; __disable_interrupt();
        ResetBitsPort(CTRL2_PORT,MASK_DRS);           //     DRS - 0
        WritePort(IODATA_DIR,0xFF);                   //    
 		  WritePort(IODATA_PORT,byte);                  //    
 		  ResetBitsPort(CTRL2_PORT,MASK_DEN);           //     DEN - 0
        SetBitsPort(CTRL2_PORT,MASK_DEN | MASK_DRS);  //     DRS - 1 DEN - 1
        WritePort(IODATA_DIR,0x00);                   //    
        SREG  = sreg;
        return;
      }
    //-------------------------------------------------------------------
    inline  void  DisplayDataWrite(unsigned char  byte)  {
      unsigned  char  sreg;

 		  sreg  = SREG; __disable_interrupt();
        WritePort(IODATA_DIR,0xFF);                   //    
        WritePort(IODATA_PORT,byte);                  //    
        ResetBitsPort(CTRL2_PORT,MASK_DEN);           //     DEN - 0
        SetBitsPort(CTRL2_PORT,MASK_DEN);             //     DEN - 1
        WritePort(IODATA_DIR,0x00);                   //    
        SREG  = sreg;
        return;
      }
    //-------------------------------------------------------------------
    inline  void  SetTxEnable(char  number) {
      unsigned  char  sreg,mask;
      
      if(number ==  0)  mask  = MASK_TXEN0;
      else              mask  = MASK_TXEN1;
      //       
      sreg  = SREG; __disable_interrupt();
      SetBitsPort(CTRL1_PORT,mask);
      //    
      SREG  = sreg;
      return;
      }
    //-------------------------------------------------------------------
    inline  void  ResetTxEnable(char  number)  {
      unsigned  char  sreg,mask;
      
      if(number ==  0)  mask  = MASK_TXEN0;
      else              mask  = MASK_TXEN1;
      //       
      sreg  = SREG; __disable_interrupt();
      ResetBitsPort(CTRL1_PORT,mask);
      //    
      SREG  = sreg;
      return;
      }
    //-------------------------------------------------------------------
    inline  unsigned  long  GetKbd(void) {
      unsigned  long  KbdScanCode;

      if(!(Debuger.blocking & LXDBL_KEY))	{
	      KbdScanCode  = (~(scan_kbd[0] ^ scan_kbd[1]) & scan_kbd[0]);
   	   KbdScanCode |= (~(scan_kbd[0] ^ scan_kbd[2]) & scan_kbd[0]);
      	KbdScanCode |= (~(scan_kbd[1] ^ scan_kbd[2]) & scan_kbd[1]);
      	}
      else	KbdScanCode	=	0xFFFFFFFFL;
		#ifdef	__DEBUGGER__
		KbdScanCode	&=	~KbdScanCode_DEB;
		#endif	//	__DEBUGGER__
      return(~KbdScanCode);
      }
    //-------------------------------------------------------------------
    inline  unsigned  int  GetDInputs(void) {
      unsigned  int DInputs,work;
      
      if(!(Debuger.blocking & LXDBL_DI))	{
	      //   8-   
   	   work  = (~(input_rk[0] ^ input_rk[1]) & input_rk[0]);
	      work |= (~(input_rk[0] ^ input_rk[2]) & input_rk[0]);
   	   work |= (~(input_rk[1] ^ input_rk[2]) & input_rk[1]);
      	#ifdef	__SMH2010_C__
	      //    -  (  C)
			DInputs	=	(work & 0x0001);
			DInputs	|=	(work & 0x0010)	<<	1;
			DInputs	|=	(work & (0x0008 | 0x0020))	>>	2;
			DInputs	|=	(work & 0x0004)	<<	4;
			DInputs	|=	(work & 0x0040)	>>	4;
			DInputs	|=	(work & 0x0080)	>>	3;
			DInputs	|=	(work & 0x0002)	<<	6;
			DInputs	|=	(work & 0x0F00);
	      #else	//	__SMH2010_C__
   	   //    -  (  )
      	DInputs  = (work         & 1) << 3;
	      DInputs |= ((work >>= 1) & 1) << 2;
   	   DInputs |= ((work >>= 1) & 1) << 1;
      	DInputs |= ((work >>= 1) & 1);
	      DInputs |= ((work >>= 1) & 1) << 5;
   	   DInputs |= ((work >>= 1) & 1) << 7;
      	DInputs |= ((work >>= 1) & 1) << 6;
	      DInputs |= ((work >>= 1) & 1) << 4;
   	   DInputs |= ((work >>= 1) & 1) << 11;
      	DInputs |= ((work >>= 1) & 1) << 8;
	      DInputs |= ((work >>= 1) & 1) << 9;
   	   DInputs |= ((work >>= 1) & 1) << 10;
      	#endif	//	__SMH2010_C__
      	}
      else	DInputs	=	0xFFFF;
		#ifdef	__DEBUGGER__
		DInputs	|=	DInputs_DEB;
		#endif	//	__DEBUGGER__
      return(DInputs);
      };
    //-------------------------------------------------------------------
    inline  void  PutDOutputs(unsigned  int data) {
		unsigned char 	work;
      if(!(Debuger.blocking & LXDBL_DI))	{
	      #ifdef	__SMH2010_C__
	      //    -  (  C)
   	   work	=	(data & 0x01)	<<	6;
      	work	|=	(data & 0x02)	<<	4;
	      work	|=	(data & 0x04)	>>	2;
   	   work	|=	(data & 0x10)	>> 1;
    		work	|=	(data & (0x08 | 0x40))	<<	1;
	      work	|=	(data & 0x20)	>>	3;
   	   work	|=	(data & 0x80)	>>	6;
	      #else	//	__SMH2010_C__
   	   //    -  (  )
      	work   =  (data         & 1);
	      work  |=  ((data >>= 1) & 1) << 1;
   	   work  |=  ((data >>= 1) & 1) << 7;
      	work  |=  ((data >>= 1) & 1) << 4;
	      work  |=  ((data >>= 1) & 1) << 2;
   	   work  |=  ((data >>= 1) & 1) << 5;
      	work  |=  ((data >>= 1) & 1) << 3;
	      work  |=  ((data >>= 1) & 1) << 6;
	      #endif	//	__SMH2010_C__
	      }
	   else	work	=	0;
		#ifdef	__DEBUGGER__
		work	|=	DInputs_DEB;
		#endif	//	__DEBUGGER__
      DOutputs = work;
      };
    //-------------------------------------------------------------------
    inline  unsigned  char  GetDOutputs(void) { return(test_rk);  }
    //-------------------------------------------------------------------
    inline  void  StartSysTimer(void)   { TCCR2 |= SYS_TIMER_PRESC;  }
    //-------------------------------------------------------------------
    inline  void  StopSysTimer(void)    { TCCR2 &= ~0x07;  }
    //-------------------------------------------------------------------
    inline  void  SetRealTime(char  hours,char  minutes,char  seconds)  {
      unsigned  char  sreg;
      char            *point;
      point = (char *)&Time;
      seconds %= 60;	minutes %= 60;	hours %= 24;
      //       
      sreg  = SREG; __disable_interrupt();
      *point++	=	seconds;	*point++	=	minutes;	*point++	=	hours;
      //    
      SREG  = sreg;
      return;
      }
    //-------------------------------------------------------------------
    inline  void  GetRealTime(char  *hours,char  *minutes,char  *seconds)  {
      unsigned  char  sreg;
      char  *point;
      point = (char *)&Time;
      //       
      sreg  = SREG; __disable_interrupt();
      *seconds  = *point++; *minutes  = *point++; *hours    = *point++;
      //    
      SREG  = sreg;
      }
    //-------------------------------------------------------------------
    inline  void  ClearRealTime(void)  {  SetRealTime(0,0,0);  }
    //-------------------------------------------------------------------
    inline void SetDays(unsigned long _days) {
      unsigned  char  sreg;
      sreg  = SREG; __disable_interrupt();
      Time.days = _days;
      SREG  = sreg;
      return;
      }
    //-------------------------------------------------------------------
    inline  unsigned  long  GetDays(void)  {
      unsigned  char  sreg;
      unsigned  long  days;
      //       
      sreg  = SREG; __disable_interrupt();
      days  = Time.days;
      //    
      SREG  = sreg;
      return  days;
      }
    //-------------------------------------------------------------------
    inline  void  IncrRealTime(void)  { //   
      char  *point;
      unsigned  long  *lpoint;
      point = (char *)&Time;
      (*point)++;
      if((*point ^ 60) == 0)  { //  
        *point++ = 0; (*point)++;
        if((*point ^ 60) == 0)  { //  
          *point++ = 0; (*point)++;
          if((*point ^ 24) == 0)  { //  
            *point++  = 0;
            lpoint  = (unsigned long  *)point;
            (*lpoint)++;
            }
          }
        }
      }
    //-------------------------------------------------------------------
    inline  int  GetSystemTicks(void) {
      unsigned  char  sreg;
                int  ticks;
      //       
      sreg  = SREG; __disable_interrupt();
      ticks  = m_lSystemTicks;
      //    
      SREG  = sreg;
      return  ticks;
      }
    //-------------------------------------------------------------------
    inline  void  EnableXRAM(void)  {
      SetBitsPort(XMCRB,0x01);    //  XMCRB |=  0x01;   //  PC7   
      SetBitsPort(DDRC,0x80);     //  DDRC  |=  0x80;   //  PC7  
      ResetBitsPort(PORTC,0x80);  //  PORTC &=  ~0x80;  //  PC7 = 0      
      }
    //-------------------------------------------------------------------
    inline  void  DisableXRAM(void) {
      SetBitsPort(XMCRB,0x01);    //  XMCRB |=  0x01;   //  PC7   
      SetBitsPort(DDRC,0x80);     //  DDRC  |=  0x80;   //  PC7  
      SetBitsPort(PORTC,0x80);    //  PORTC |=  0x80;   //  PC7 = 1      
      }
    //-------------------------------------------------------------------
    inline  void  EnableWDT(unsigned	char	period) {
    	unsigned	char	sreg;
 		//	  watchdog     
  		MCUCSR &= ~(1 << WDRF);
  		//	 watchdog
    	sreg	=	SREG;	__disable_interrupt();
    	WDTCR	= 	(1 << WDCE)	|	(1 << WDE);
    	WDTCR	= 	(1 << WDE)	|	period;
    	SREG	=	sreg;
    	}
    //-------------------------------------------------------------------
    inline  void  DisableWDT(void) {
    	unsigned	char	sreg;
 		//	  watchdog     
  		MCUCSR &= ~(1 << WDRF);
		//	 watchdog
    	sreg	=	SREG;	__disable_interrupt();
    	WDTCR		= 	(1 << WDCE)	|	(1 << WDE);
    	WDTCR		=	0x00;
    	SREG	=	sreg;
    	}
    //-------------------------------------------------------------------
    inline  void  ResetWDT(void) {	__watchdog_reset();	}
#ifdef	__SMH2010_C__
    //-------------------------------------------------------------------
    inline  void  LightON(void)		{	SetBitsPort(CTRL1_PORT,MASK_LIGHT);	}
    //-------------------------------------------------------------------
    inline  void  LightOFF(void)		{	ResetBitsPort(CTRL1_PORT,MASK_LIGHT);	}
    //-------------------------------------------------------------------
    inline  void  LightTOGGLE(void)	{	ToggleBitsPort(CTRL1_PORT,MASK_LIGHT);	}
#endif	//	__SMH2010_C__
};
extern  ClassBiosSMH    *Bios;
#endif  //  __SMH1010_HPP_
