/****************************************************************************
 **             - iom128.h -
 **
 **     This file declares the internal register addresses for ATmega128.
 **
 **     Used with iccAVR and aAVR.
 **
 **     Copyright IAR Systems 1999. All rights reserved.
 **
 **     $Name: V2.21H V2.21G V2_25A V2_21B $
 **
 ***************************************************************************/
#include "iomacro.h"
#include "iomacros.h"

#if TID_GUARD(3)
#error This file should only be compiled with iccavr or aavr whith processor option -v3
#endif /* TID_GUARD(3) */

#ifndef __IOM128_H
#define __IOM128_H

#pragma language=extended
/*==========================*/
/* Predefined SFR Addresses */
/*==========================*/

SFR_B_128(PINF,   0x20) /* Input Pins, Port F */
SFR_B_128(PINE,   0x21) /* Input Pins, Port E */
SFR_B_128(DDRE,   0x22) /* Data Direction Register, Port E */
SFR_B_128(PORTE,  0x23) /* Data Register, Port E */
SFR_W_128(ADC,    0x24) /* ADC Data register */
SFR_B_128(ADCSRA, 0x26) /* ADC Control and Status Register */
SFR_B_128(ADMUX,  0x27) /* ADC MUX */
SFR_B_128(ACSR,   0x28) /* Analog Comparator Control and Status Register */
SFR_B_128(UBRR0L, 0x29) /* UART Baud Rate Register */
SFR_B_128(UCSR0B, 0x2A) /* UART Control Register */
SFR_B_128(UCSR0A, 0x2B) /* UART Status Register */
SFR_B_128(UDR0,   0x2C) /* UART I/O Data Register */
SFR_B_128(SPCR,   0x2D) /* SPI Control Register */
SFR_B_128(SPSR,   0x2E) /* SPI Status Register */
SFR_B_128(SPDR,   0x2F) /* SPI I/O Data Register */
SFR_B_128(PIND,   0x30) /* Input Pins, Port D */
SFR_B_128(DDRD,   0x31) /* Data Direction Register, Port D */
SFR_B_128(PORTD,  0x32) /* Data Register, Port D */
SFR_B_128(PINC,   0x33) /* Input Pins, Port C */
SFR_B_128(DDRC,   0x34) /* Data Direction Register, Port C */
SFR_B_128(PORTC,  0x35) /* Data Register, Port C */
SFR_B_128(PINB,   0x36) /* Input Pins, Port B */
SFR_B_128(DDRB,   0x37) /* Data Direction Register, Port B */
SFR_B_128(PORTB,  0x38) /* Data Register, Port B */
SFR_B_128(PINA,   0x39) /* Input Pins, Port A */
SFR_B_128(DDRA,   0x3A) /* Data Direction Register, Port A */
SFR_B_128(PORTA,  0x3B) /* Data Register, Port A */
SFR_B_128(EECR,   0x3C) /* EEPROM Control Register */
SFR_B_128(EEDR,   0x3D) /* EEPROM Data Register */
SFR_W_128(EEAR,   0x3E) /* EEPROM Address Register */
SFR_B_128(SFIOR,  0x40)	/* SFIOR */
SFR_B_128(WDTCR,  0x41) /* Watchdog Timer Control Register */
SFR_B_128(OCR2,   0x43) /* Timer/Counter2 Output Compare Register */
SFR_B_128(TCNT2,  0x44) /* Timer/Counter 2 */
SFR_B_128(TCCR2,  0x45) /* Timer/Counter 2 Control Register */
SFR_W_128(ICR1,   0x46) /* T/C 1 Input Capture Register */
SFR_W_128(OCR1B,  0x48) /* Timer/Counter1 Output Compare Register B */
SFR_W_128(OCR1A,  0x4A) /* Timer/Counter1 Output Compare Register A */
SFR_W_128(TCNT1,  0x4C) /* Timer/Counter 1 */
SFR_B_128(TCCR1B, 0x4E) /* Timer/Counter 1 Control and Status Register */
SFR_B_128(TCCR1A, 0x4F) /* Timer/Counter 1 Control Register */
SFR_B_128(ASSR,   0x50) /* Timer/Counter 0 Asynchronous Control & Status Register */
SFR_B_128(OCR0,   0x51) /* Output Compare Register 0 */
SFR_B_128(TCNT0,  0x52) /* Timer/Counter 0 */
SFR_B_128(TCCR0,  0x53) /* Timer/Counter 0 Control Register */
SFR_B_128(MCUSR,  0x54) /* MCU Status Register */
SFR_B_128(MCUCR,  0x55) /* MCU general Control Register */
SFR_B_128(TIFR,   0x56) /* Timer/Counter Interrupt Flag register */
SFR_B_128(TIMSK,  0x57) /* Timer/Counter Interrupt MaSK register */
SFR_B_128(EIFR,   0x58) /* xternal Interrupt Flag Register */
SFR_B_128(EIMSK,  0x59) /* External Interrupt MaSK register */
SFR_B_128(EICRB,  0x5A) /* External Interrupt Control Register */
SFR_B_128(RAMPZ,  0x5B) /* RAM Page Z select register */
SFR_B_128(XDIV,   0x5C) /* XDIV Divide control register */
SFR_W_128(SP,     0x5D) /* Stack Pointer */
SFR_B_128(SREG,   0x5F) /* Status REGister */

#define	MCUCSR	MCUSR

SFR_B_128(DDRF,  0x61)
SFR_B_128(PORTF, 0x62)
SFR_B_128(PING,  0x63)
SFR_B_128(DDRG,  0x64)
SFR_B_128(PORTG, 0x65)
SFR_B_128(SPMCSR,0x68)
SFR_B_128(EICRA, 0x6A)
SFR_B_128(XMCRB, 0x6C)
SFR_B_128(XMCRA, 0x6D)
SFR_B_128(OSCCAL,0x6F)
SFR_B_128(TWBR,  0x70)
SFR_B_128(TWSR,  0x71)
SFR_B_128(TWAR,  0x72)
SFR_B_128(TWDR,  0x73)
SFR_B_128(TWCR,  0x74)
SFR_W_128(OCR1C, 0x78)
SFR_B_128(TCCR1C,0x7A)
SFR_B_128(ETIFR, 0x7C)
SFR_B_128(ETIMSK,0x7D)
SFR_W_128(ICR3,  0x80)
SFR_W_128(OCR3C, 0x82)
SFR_W_128(OCR3B, 0x84)
SFR_W_128(OCR3A, 0x86)
SFR_W_128(TCNT3, 0x88)
SFR_B_128(TCCR3B,0x8A)
SFR_B_128(TCCR3A,0x8B)
SFR_B_128(TCCR3C,0x8C)
SFR_B_128(UBRR0H,0x90)
SFR_B_128(UCSR0C,0x95)
SFR_B_128(UBRR1H,0x98)
SFR_B_128(UBRR1L,0x99)
SFR_B_128(UCSR1B,0x9A)
SFR_B_128(UCSR1A,0x9B)
SFR_B_128(UDR1,  0x9C)
SFR_B_128(UCSR1C,0x9D)

/*========================================*/
/* Interrupt Vector Definitions with 128  */
/*========================================*/
/* NB! vectors are specified as byte addresses */
 
#define RESET_vect        (0x00)
#define INT0_vect         (0x04)
#define INT1_vect         (0x08)
#define INT2_vect         (0x0C)
#define INT3_vect         (0x10)
#define INT4_vect         (0x14)
#define INT5_vect         (0x18)
#define INT6_vect         (0x1C)
#define INT7_vect         (0x20)
#define TIMER2_COMP_vect  (0x24)
#define TIMER2_OVF_vect   (0x28)
#define TIMER1_CAPT_vect  (0x2C)
#define TIMER1_COMPA_vect (0x30)
#define TIMER1_COMPB_vect (0x34)
#define TIMER1_OVF_vect   (0x38)
#define TIMER0_COMP_vect  (0x3C)
#define TIMER0_OVF_vect   (0x40)
#define SPI_STC_vect      (0x44)
#define USART0_RX_vect    (0x48)
#define USART0_UDRE_vect  (0x4C)
#define USART0_TX_vect    (0x50)
#define ADC_vect          (0x54)
#define EE_RDY_vect       (0x58)
#define ANA_COMP_vect     (0x5C)
#define TIMER1_COMPC_vect (0x60)
#define TIMER3_CAPT_vect  (0x64)
#define TIMER3_COMPA_vect (0x68)
#define TIMER3_COMPB_vect (0x6C)
#define TIMER3_COMPC_vect (0x70)
#define TIMER3_OVF_vect   (0x74)
#define USART1_RX_vect    (0x78)
#define USART1_UDRE_vect  (0x7C)
#define USART1_TX_vect    (0x80)
#define TWI_vect          (0x84)
#define SPM_READY_vect    (0x88)

#ifdef __IAR_SYSTEMS_ASM__
#ifndef ENABLE_BIT_DEFINITIONS
#define  ENABLE_BIT_DEFINITIONS
#endif /* ENABLE_BIT_DEFINITIONS */
#endif /* __IAR_SYSTEMS_ASM__ */

#ifdef ENABLE_BIT_DEFINITIONS

/* Bit definitions for use with the IAR Assembler   
   The Register Bit names are represented by their bit number (0-7).
*/                                                


#define	PUD	2
#define	SPMIE	7
#define	RWWSB	6
#define	RWWSRE	4
#define	BLBSET	3
#define	PGWRT	2
#define	PGERS	1
#define	SPMEN	0

/* Bits for two wire interface */
#define	TWBR0	0
#define	TWBR1	1
#define	TWBR2	2
#define	TWBR3	3
#define	TWBR4	4
#define	TWBR5	5
#define	TWBR6	6
#define	TWBR7	7

#define	TWIE	0
#define	TWEN	2
#define	TWWC	3
#define	TWSTO	4
#define	TWSTA	5
#define	TWEA	6
#define	TWINT	7

#define	TWPS0	0
#define	TWPS1	1
#define	TWS3	3
#define	TWS4	4
#define	TWS5	5
#define	TWS6	6
#define	TWS7	7

#define	TWGCE	0
#define	TWA0	1
#define	TWA1	2
#define	TWA2	3
#define	TWA3	4
#define	TWA4	5
#define	TWA5	6
#define	TWA6	7

#define	TOV3	2


/* XDIV Divide control register*/
#define    XDIVEN       7
#define    XDIV6        6
#define    XDIV5        5
#define    XDIV4        4
#define    XDIV3        3
#define    XDIV2        2
#define    XDIV1        1
#define    XDIV0        0
 
/* RAM Page Z select register */
#define     RAMPZ0      0
 
/* External Interrupt Control Register */ 
#define    ISC71        7
#define    ISC70        6
#define    ISC61        5
#define    ISC60        4
#define    ISC51        3
#define    ISC50        2
#define    ISC41        1
#define    ISC40        0
 
/* External Interrupt MaSK register */
#define    INT7         7
#define    INT6         6   
#define    INT5         5
#define    INT4         4
#define    INT3         3
#define    INT2         2
#define    INT1         1
#define    INT0         0
 
/* xternal Interrupt Flag Register */
#define    INTF7        7
#define    INTF6        6
#define    INTF5        5
#define    INTF4        4
#define    INTF3        3
#define    INTF2        2
#define    INTF1        1
#define    INTF0        0
 
/* Timer/Counter Interrupt MaSK register */ 
#define    OCIE2        7
#define    TOIE2        6
#define    TICIE1       5
#define    OCIE1A       4
#define    OCIE1B       3
#define    TOIE1        2
#define    OCIE0        1
#define    TOIE0        0
 
/* Timer/Counter Interrupt Flag Register */
#define    OCF2         7
#define    TOV2         6
#define    ICF1         5
#define    OCF1A        4
#define    OCF1B        3
#define    TOV1         2
#define    OCF0         1
#define    TOV0         0      
 
/* MCU general Control Register */
#define    SRE          7
#define    SRW          6
#define    SE           5
#define    SM1          4
#define    SM0          3
#define    SM2          2
 
/* MCU Status Register */ 
#define    JTRF        	4
#define    WDRF        	3
#define    BORF        	2
#define    EXTRF        1
#define    PORF         0
 
/* Timer/Counter 0 Control Register */
#define    PWM0         6
#define    COM01        5
#define    COM00        4
#define    CTC0         3
#define    CS02         2
#define    CS01         1
#define    CS00         0
#define    WGM20        PWM0
#define    WGM21        CTC0
 
/* Timer/Counter 0 Asynchronous Control & Status Register */ 
#define    AS0          3
#define    TCN0UB       2
#define    OCR0UB       1
#define    TCR0UB       0 
 
/* Timer/Counter 1 Control Register (A) */ 
#define    COM1A1       7
#define    COM1A0       6
#define    COM1B1       5
#define    COM1B0       4
#define    PWM11        1
#define    PWM10        0

#define    COM1C1       3
#define    COM1C0       2
#define    WGM11        PWM11
#define    WGM10        PWM10

 
/* Timer/Counter 1 Control and Status Register (B) */
#define    ICNC1        7
#define    ICES1        6
#define    CTC1         3
#define    CS12         2
#define    CS11         1
#define    CS10         0
 
#define    WGM13        4
#define    WGM12        CTC1


/* Timer/Counter 2 Control register */ 
#define    FOC2         7
#define    PWM2         6
#define    COM21        5
#define    COM20        4
#define    CTC2         3
#define    CS22         2
#define    CS21         1
#define    CS20         0
#define    WGM00        PWM2
#define    WGM01        CTC2
 
/* Watchdog Timer Control Register */ 
#define    WDTOE        4
#define    WDE          3
#define    WDP2         2
#define    WDP1         1
#define    WDP0         0
#define    WDCE		4
  
/* EEPROM Control Register */
#define    EERIE        3
#define    EEMWE        2
#define    EEWE         1
#define    EERE         0
 
/* Data Register, Port A */
#define    PA7          7
#define    PA6          6
#define    PA5          5
#define    PA4          4
#define    PA3          3
#define    PA2          2
#define    PA1          1
#define    PA0          0
 
/* Data Direction Register, Port A */ 
#define    DDA7         7
#define    DDA6         6
#define    DDA5         5
#define    DDA4         4
#define    DDA3         3
#define    DDA2         2
#define    DDA1         1
#define    DDA0         0
   
/* Input Pins, Port A */ 
#define    PINA7        7
#define    PINA6        6
#define    PINA5        5
#define    PINA4        4
#define    PINA3        3
#define    PINA2        2
#define    PINA1        1
#define    PINA0        0
 
/* Data Register, Port B */ 
#define    PB7          7
#define    PB6          6
#define    PB5          5
#define    PB4          4
#define    PB3          3
#define    PB2          2
#define    PB1          1
#define    PB0          0
 
/* Data Direction Register, Port B */
#define    DDB7         7
#define    DDB6         6
#define    DDB5         5
#define    DDB4         4
#define    DDB3         3
#define    DDB2         2
#define    DDB1         1
#define    DDB0         0
                      
/* Input Pins, Port B */
#define    PINB7        7
#define    PINB6        6
#define    PINB5        5
#define    PINB4        4
#define    PINB3        3
#define    PINB2        2
#define    PINB1        1
#define    PINB0        0
 
/* Data Register, Port C */
#define    PC7          7
#define    PC6          6
#define    PC5          5
#define    PC4          4
#define    PC3          3
#define    PC2          2
#define    PC1          1
#define    PC0          0
 
/* Data Register, Port D */
#define    PD7          7
#define    PD6          6
#define    PD5          5
#define    PD4          4
#define    PD3          3
#define    PD2          2
#define    PD1          1
#define    PD0          0
                                          
/* Data Direction Register, Port D */
#define    DDD7         7
#define    DDD6         6
#define    DDD5         5
#define    DDD4         4
#define    DDD3         3
#define    DDD2         2
#define    DDD1         1
#define    DDD0         0
 
/* Input Pins, Port D */
#define    PIND7        7
#define    PIND6        6
#define    PIND5        5
#define    PIND4        4
#define    PIND3        3
#define    PIND2        2
#define    PIND1        1
#define    PIND0        0
 
/* Data Register, Port E */
#define    PE7          7
#define    PE6          6
#define    PE5          5
#define    PE4          4
#define    PE3          3
#define    PE2          2
#define    PE1          1
#define    PE0          0
 
/* Data Direction Register, Port E */
#define    DDE7         7
#define    DDE6         6
#define    DDE5         5
#define    DDE4         4
#define    DDE3         3
#define    DDE2         2
#define    DDE1         1
#define    DDE0         0
 
/* Input Pins, Port E */
#define    PINE7        7
#define    PINE6        6
#define    PINE5        5
#define    PINE4        4
#define    PINE3        3
#define    PINE2        2
#define    PINE1        1
#define    PINE0        0
 
/* Input Pins, Port F */
#define    PINF7        7
#define    PINF6        6
#define    PINF5        5
#define    PINF4        4
#define    PINF3        3
#define    PINF2        2
#define    PINF1        1
#define    PINF0        0
 
/* SPI Status Register */ 
#define    SPIF         7
#define    WCOL         6
 
/* SPI Control Register */
#define    SPIE         7
#define    SPE          6
#define    DORD         5
#define    MSTR         4
#define    CPOL         3
#define    CPHA         2
#define    SPR1         1
#define    SPR0         0
  
/* UART Control and Status Register A */
#define    RXC          7
#define    TXC          6
#define    UDRE         5
#define    FE           4
#define    DOR          3
#define    UPE          2
#define    U2X          1
#define    MPCM         0
 
/* UART Control and Status Register B */
#define    RXCIE        7
#define    TXCIE        6
#define    UDRIE        5
#define    RXEN         4
#define    TXEN         3
#define    UCSZ2        2
#define    RXB8         1
#define    TXB8         0

/* UART Control and Status Register C */
#define    UMSEL        6
#define    UPM1         5
#define    UPM0         4
#define    USBS         3
#define    UCSZ1        2
#define    UCSZ0        1
#define    UCPOL        0
 
/* Analog Comparator Control and Status Register */
#define    ACD          7
#define    ACO          5
#define    ACI          4
#define    ACIE         3
#define    ACIC         2
#define    ACIS1        1
#define    ACIS0        0
 
/* ADC Control and status register */
#define    ADEN         7
#define    ADSC         6
#define    ADFR         5
#define    ADIF         4
#define    ADIE         3
#define    ADPS2        2
#define    ADPS1        1
#define    ADPS0        0
 
/* ADC Multiplexer select */
#define    ADLAR        5
#define    MUX4         4
#define    MUX3         3
#define    MUX2         2
#define    MUX1         1
#define    MUX0         0
 
/* Pointer definition */
#define    XL       r26
#define    XH       r27
#define    YL       r28
#define    YH       r29
#define    ZL       r30
#define    ZH       r31

/* Constants */
#define    RAMEND   0x0FFF    /* Last On-Chip SRAM Location */
#define    XRAMEND  0xFFFF
#define    E2END    0x0FFF
#define    FLASHEND 0x1FFFF
 
#endif /* ENABLE_BIT_DEFINITIONS */
#endif /* __IOM128_H (define part) */

#pragma language=default
