#ifndef __SMHCOM_HPP__
#define __SMHCOM_HPP__

#include  <iom128.h>
#include  <stdlib.h>
#include  <string.h>
#include  <smh2010.hpp>

#define RS_NUMBER_EMPTY_BYTE  12

//#define SYSCLCK_PER_RSBYTE  24
#define COM_MAXNUMBER   	 2
#define COM_MAXBUFFERSIZE   (128 + 9)	//	32

#define BAUD_230400	3
#define BAUD_115200	7
#define BAUD_76800	11
#define BAUD_57600	15
#define BAUD_38400	23
#define BAUD_28800	31
#define BAUD_19200	47
#define BAUD_14400	63
#define BAUD_09600	95
#define BAUD_04800	191

#define PRTY_NONE     0
#define PRTY_ODD      1
#define PRTY_EVEN     2

#define RSFIVE_BITS   0
#define RSSIX_BITS    1
#define RSSEVEN_BITS  2
#define RSEIGHT_BITS  3

#define RSSTOP_1BIT   0
#define RSSTOP_2BIT   1

//	  
#define	RSRECS_NONE			0x00	//	start-false,	stop-false
#define	RSRECS_START      0x01	//	start-true,		stop-false
#define	RSRECS_STOP       0x02	//	start-false,	stop-true
#define	RSRECS_START_STOP	0x04	//	start-true,		stop-true

class ClassComs {
  //------------------------------------------------------------------------
  private:
    volatile            bool    number; //  IF == true THEN Com1 (fullduplex)
    volatile            bool    selftransmitt;
    volatile            bool    halfduplex,busy;
    volatile  unsigned  char    *UBRRL,*UBRRH,*UDR,*UCSRA,*UCSRB,*UCSRC;
    volatile            char    trm_state;  //         4-  
    volatile            int     trm_cntr,trm_msgsize,trm_maxsize;
    volatile            char    rec_state;
    volatile            int     rec_cntr,rec_msgsize,rec_maxsize;
    volatile  unsigned  char    *trm_buffer,*rec_buffer;
    volatile  unsigned  int     m_oldclck;    //       
                                              //     (  )
    volatile  unsigned  char    currAddress;

//------------------------------------------------------------------------
    inline  void  start_transmitt(void)  {
		if(number)
		  #ifdef __OPTO485__
  		        start_transmitt_485();
  		  #else
		        start_transmitt_232();
		  #endif	
		else
		  start_transmitt_485();
    	}
    //------------------------------------------------------------------------
    //	   4         
    inline  void  start_transmitt_485(void)  {
      if(trm_msgsize <= 0)
        return;
      if(halfduplex)  {
        selftransmitt = true;
        //        
        *UCSRB  &=  ~(1 << RXCIE);
        }
      trm_state   = trm_cntr  = 0;
      Transmitted = false;
      *UCSRB      |=  (1 << TXCIE);
      *UDR        = 0x00;
      }
    //------------------------------------------------------------------------
    inline  void  start_transmitt_232(void)  {
      if(trm_msgsize <= 0)
         return;
      if(halfduplex)  {
        selftransmitt = true;
        //        
        *UCSRB  &=  ~(1 << RXCIE);
        }
      trm_state   = trm_cntr  = 0;
      Transmitted = false;
		Bios->SetTxEnable((char)number);
      *UCSRB      |=  (1 << TXCIE);
		*UDR  = trm_buffer[trm_cntr++];
      }

    //------------------------------------------------------------------------
    inline  void  stop_transmitt(void) {
    volatile  unsigned  char  wrkchar;
    
      if(halfduplex)  {
        selftransmitt = false;
        //  H   
        rec_state = RSRECS_START;
        //    com- (FIFO)
        wrkchar = *UDR;
        wrkchar = *UDR;
        wrkchar = *UDR;
        //     
        *UCSRB  |=  (1 << RXCIE);
        }
      Transmitted  = true;
      }
    //------------------------------------------------------------------------
  public:
    ClassBiosSMH              *Bios;
    unsigned	char				numb_prot;
    volatile  bool            Transmitted,Recived;
    volatile  unsigned  int   RecByteTimeOut,FrameTimeOut;    //     
    volatile  unsigned  int   ERecByteTimeOut,ERecByteFrame,ERecParity,ERecByteOverRun;
    //------------------------------------------------------------------------
    ClassComs(int comnumber) :  busy(false),	numb_prot(0),
                                rec_state(RSRECS_NONE),  rec_cntr(0),    	rec_msgsize(0), 	rec_maxsize(0), rec_buffer(NULL),	Recived(false),
                                trm_state(0),   			trm_cntr(0),    	trm_msgsize(0), 	trm_maxsize(0), trm_buffer(NULL),	Transmitted(true), 
                                ERecByteTimeOut(0), 		ERecByteFrame(0),	ERecParity(0),		ERecByteOverRun(0), currAddress(0)
                                {
      if(number = (bool)comnumber) {
        UBRRL = (unsigned  char *)&UBRR1L;  UBRRH = (unsigned  char *)&UBRR1H;
        UDR   = (unsigned  char *)&UDR1;    UCSRA = (unsigned  char *)&UCSR1A;
        UCSRB = (unsigned  char *)&UCSR1B;  UCSRC = (unsigned  char *)&UCSR1C;
        #ifdef __OPTO485__
            halfduplex = true;
        #else    
            halfduplex = false;
        #endif    
        }
      else  {
        UBRRL = (unsigned  char *)&UBRR0L;  UBRRH = (unsigned  char *)&UBRR0H;
        UDR   = (unsigned  char *)&UDR0;    UCSRA = (unsigned  char *)&UCSR0A;
        UCSRB = (unsigned  char *)&UCSR0B;  UCSRC = (unsigned  char *)&UCSR0C;
        halfduplex  = true;
        }
        UBRR1H	=	UBRR0H	=	0;
      }
    //------------------------------------------------------------------------
    ~ClassComs(void) {
      free((void  *)rec_buffer);
      free((void  *)trm_buffer);
      }
    //------------------------------------------------------------------------
    inline  void  Sleep(void) {
      //     
      *UCSRB  &=  ~((1 << TXEN) | (1 << RXEN));
      //     
      *UCSRB  &=  ~(1 << RXCIE);
      }
    //------------------------------------------------------------------------
    inline  void  WakeUp(void)  {
      //     
      *UCSRB  |=  ((1 << TXEN) | (1 << RXEN));
      //     
      *UCSRB  |=  (1 << RXCIE);
      }
//    //------------------------------------------------------------------------
//    inline  __monitor bool  Lock(void)    {
//      if(!IsLock()) { busy  = true; return  true; }
//      return  false;
//      }
//    //------------------------------------------------------------------------
//    inline  __monitor void  UnLock(void)          { busy  = false; }
//    //------------------------------------------------------------------------
//    inline  bool	IsLock(void)                    { return  busy;}
    //------------------------------------------------------------------------
    inline  unsigned char	GetNumber()                     { return  (unsigned char)number; }
    //------------------------------------------------------------------------
    inline  unsigned  char  *PointToRecBuffer()   { return  (unsigned  char  *)rec_buffer; }
    //------------------------------------------------------------------------
    inline  unsigned  char  *PointToTrmBuffer()   { return  (unsigned  char  *)trm_buffer; }
    //------------------------------------------------------------------------
    inline  int	GetSizeRecMessage(void)         { return  rec_msgsize;  }
    //------------------------------------------------------------------------
    inline  char  GetRecState(void)               { return  rec_state;    }
    //------------------------------------------------------------------------
    inline  void  FlushReciver()                  { unsigned char data = *UDR;	Recived = false;  }
    //------------------------------------------------------------------------
    inline  void  FlushTransmitter()              { stop_transmitt();	}
    //------------------------------------------------------------------------
    inline  void  Flush()                         {  FlushReciver(); FlushTransmitter();  }
    //------------------------------------------------------------------------
    inline  void	SetCurrAddress(unsigned char adr)  { currAddress =  adr; }
    //------------------------------------------------------------------------
    bool  SetRecBuffer(int  newsize)  {
	   rec_maxsize = 0;
	   FlushReciver();
		if(NULL	!=	(rec_buffer	=	(volatile  unsigned  char  *)realloc((void	*)rec_buffer,(size_t)(newsize + 1))))	{
      	rec_maxsize = newsize;
        	return  true;
			}
  		return  false;
      }
    //------------------------------------------------------------------------
    bool  SetTrmBuffer(int  newsize)  {
     	trm_maxsize = 0;
      FlushTransmitter();
		if(NULL	!=	(trm_buffer	=	(volatile  unsigned  char  *)realloc((void	*)trm_buffer,(size_t)(newsize + 1))))	{
      	trm_maxsize = newsize;
        	return  true;
			}
     	return  false;
      }
    //------------------------------------------------------------------------
    inline	void  Init(unsigned char  baud)  {
      //	       mega128 - 8,  , 1  
      SetBaudRate(baud);
     	//    1- 
      SetFrameTimeOut(4);
     	//     
      SetByteTimeOut(3);
      SetRecBuffer(COM_MAXBUFFERSIZE);
      Flush();
      //     
      *UCSRB  |=  ((1 << TXEN) | (1 << RXEN));
      //     
      *UCSRB  |=  (1 << RXCIE);
      return;
    }
   //------------------------------------------------------------------------
	//	   4800    14Hz
    inline	void  SetBaudRate(unsigned char  baud)  {
    	switch(baud)	{
			case	BAUD_230400:	case	BAUD_76800:	case	BAUD_57600:	case	BAUD_38400:
			case	BAUD_28800:		case	BAUD_19200:	case	BAUD_14400:	case	BAUD_09600:
			case	BAUD_04800:		*UBRRL  = baud;	break;
			default:
			case	BAUD_115200:	*UBRRL  = BAUD_115200;	break;
			}
    	}
   //------------------------------------------------------------------------
    inline	unsigned	int	CalcSysClckPerByte(void)	{
    	//	   ()
    	switch(*UBRRL)	{
			case	BAUD_230400:	return((unsigned	int)(11.5 / 0.2304	/	TIMER3_FltTICK));
			case	BAUD_76800:		return((unsigned	int)(11.5 / 0.0768	/	TIMER3_FltTICK));
			case	BAUD_57600:		return((unsigned	int)(11.5 / 0.0576	/	TIMER3_FltTICK));
			case	BAUD_38400:		return((unsigned	int)(11.5 / 0.0384	/	TIMER3_FltTICK));
			case	BAUD_28800:		return((unsigned	int)(11.5 / 0.0288	/	TIMER3_FltTICK));
			case	BAUD_19200:		return((unsigned	int)(11.5 / 0.0192	/	TIMER3_FltTICK));
			case	BAUD_14400:		return((unsigned	int)(11.5 / 0.0144	/	TIMER3_FltTICK));
			case	BAUD_09600:		return((unsigned	int)(11.5 / 0.0096	/	TIMER3_FltTICK));
			case	BAUD_04800:		return((unsigned	int)(11.5 / 0.0048	/	TIMER3_FltTICK));
			default:
			case	BAUD_115200:	return((unsigned	int)(11.5 / 0.1152	/	TIMER3_FltTICK));
    		}
    	}
    //------------------------------------------------------------------------
    //																					  
    void  SetByteTimeOut(unsigned char  numb)	{	RecByteTimeOut  = (CalcSysClckPerByte() >> 1) * (numb + 1);  }
    //------------------------------------------------------------------------
    void  SetFrameTimeOut(unsigned char  numb)	{	FrameTimeOut  = CalcSysClckPerByte() * (numb + 1);  }
    //------------------------------------------------------------------------
    void  SetParity(unsigned char  prty)  {
      //     
      *UCSRC  &=  ~((1 << UPM0) | (1 << UPM1)); 
      switch(prty)  {
        default:	case  PRTY_NONE:  break;
        case  PRTY_ODD:   *UCSRC  |=  (1 << UPM0);
        case  PRTY_EVEN:  *UCSRC  |=  (1 << UPM1);  break;
        }
      }
    //------------------------------------------------------------------------
    void  SetStopBits(unsigned char  sb)  {
      *UCSRC  &=  ~(1 << USBS);
      if(sb == RSSTOP_2BIT)
        *UCSRC  |=  (1 << USBS);
      }
    //------------------------------------------------------------------------
    void  SetBits(unsigned char  numb)  {
      //  
      *UCSRC  &=  ~(RSEIGHT_BITS << UCSZ0);
      //  
      *UCSRC  |=  ((numb & RSEIGHT_BITS) << UCSZ0);
      }
    //------------------------------------------------------------------------
    inline  void  WriteNextByte(void)	{
      if(number)
	  #ifdef __OPTO485__
                 WriteNextByte_485();	  
          #else       
                 WriteNextByte_232();
          #endif       

		else
		  WriteNextByte_485();
		}
    //------------------------------------------------------------------------
    //	  9         
    inline  void  WriteNextByte_485(void)  {
    volatile  unsigned  char  wrkchar;
    
    //      
    if(trm_state   <   (RS_NUMBER_EMPTY_BYTE - 1))	{
      *UDR  = 0x00;
      trm_state++;
      return;
      }
    if(trm_state   ==  (RS_NUMBER_EMPTY_BYTE - 1))
      Bios->SetTxEnable((char)number);
    if(trm_cntr >= trm_msgsize) {
      //      
      m_oldclck = TCNT3;
      Bios->ResetTxEnable((char)number);
      stop_transmitt();
      return;
      }
    *UDR  = trm_buffer[trm_cntr++];
    return;
    }
    //------------------------------------------------------------------------
    inline  void  WriteNextByte_232(void)  {
    volatile  unsigned  char  wrkchar;
		
    if(trm_cntr >= trm_msgsize)	{
      stop_transmitt();
      Bios->ResetTxEnable((char)number);
      }
    else  
      *UDR  = trm_buffer[trm_cntr++];
    return;
    }


    //------------------------------------------------------------------------
    //   modbus (   )
    inline  void  TestFrame(void) {
    unsigned  int   tcnt3;
    unsigned  char  sreg;
    //       
    sreg  = SREG; __disable_interrupt();
    //   - 
    tcnt3 = TCNT3;
    tcnt3	-=	m_oldclck;
    if(tcnt3 < m_oldclck) tcnt3  +=  0xFFFF;
	 //	  -     
    if(FrameTimeOut <= tcnt3)	{
		if(RSRECS_NONE == rec_state)  
		  rec_state = RSRECS_STOP;
		else	if(RSRECS_STOP == rec_state)
		  rec_state = RSRECS_START;
		else	if(RSRECS_START_STOP == rec_state)	{
					rec_state		=	RSRECS_STOP;
					rec_msgsize   	= rec_cntr;
					Recived       	= true;
					}
		 }
    //    
    SREG      = sreg;
    return;
    }

    //------------------------------------------------------------------------
    inline  void  ReadNextByte(void)  {
      volatile  bool  timeout;
      unsigned  char  ctrla,data;
      unsigned  int   tcnt3,work;

      ctrla     = *UCSRA;
      data      = *UDR;
      tcnt3     = TCNT3;
      
      //   -  
      work	=	tcnt3	-	m_oldclck;
      if(work < m_oldclck) work  +=  0xFFFF;
      timeout = true;
      if(RecByteTimeOut > work) timeout = false;
      //   frame MODBUS
      TestFrame();
      //      
      m_oldclck = tcnt3;

      //       
      if((rec_state  ==  RSRECS_START)  ||  (rec_state  ==  RSRECS_START_STOP))  {
        //   
        if(ctrla & (1 << UPE))  {
          rec_state = RSRECS_NONE;  ERecParity++;
          }
        //    
        if(ctrla & (1 << FE))   {
          rec_state = RSRECS_NONE;  ERecByteFrame++;
          }
        //     (!!!  rec_state = RSRECS_START    !!!)
        if((RSRECS_START_STOP == rec_state) && (ctrla & (1 << DOR)))  {
          rec_state = RSRECS_NONE;  ERecByteOverRun++;
          }
        }
		//	 
		if(RSRECS_START == rec_state)	{
         if(!Recived)  {
            if(data == currAddress) {
              rec_state = RSRECS_START_STOP;
              rec_cntr  = 0;
              //     
              rec_buffer[rec_cntr++]  = data;
              }
            else  rec_state	=	RSRECS_NONE;
            }
			else	rec_state	=	RSRECS_NONE;
			}
		//	 
		else	if(RSRECS_START_STOP == rec_state)	{
      	if(timeout) {
         	rec_state = RSRECS_NONE;
            ERecByteTimeOut++;
            }
         rec_buffer[rec_cntr++]  = data;
         if(rec_cntr >= rec_maxsize) {
         	rec_state = RSRECS_STOP;
            rec_msgsize   = rec_cntr;                         
            Recived       = true;
            }
			}
      return;
      }
    //------------------------------------------------------------------------
    bool  SetMessage(int len,unsigned char  *point) {
      if(!Transmitted || (len <=  0))
        return	false;
      if(!SetTrmBuffer(len))
        return  false;
      trm_msgsize = len;
		memcpy((void *)trm_buffer,(void *)point,(size_t)len);
      start_transmitt();
      return  true;
      }
    //------------------------------------------------------------------------
    int GetMessage(unsigned char  *buff) {
		memcpy((void *)buff,(void *)rec_buffer,(size_t)rec_msgsize);
      Recived = false;
      return  rec_msgsize;
      }
    //------------------------------------------------------------------------
    inline  void ClrErrors(void) {
      ERecByteTimeOut = ERecByteFrame = ERecParity  =  ERecByteOverRun  = 0;
      }
    //------------------------------------------------------------------------
    inline  bool isErrors(void) {
      if(     (0  !=  ERecByteTimeOut)\
          ||  (0  !=  ERecByteFrame)\
          ||  (0  !=  ERecParity)\
          ||  (0  !=  ERecByteOverRun))  return  true;
      return  false;
      }
    //------------------------------------------------------------------------
    inline  unsigned	long GetErrors(void) {
    	return(		(unsigned	long)ERecByteTimeOut\
    				+	(unsigned	long)ERecByteFrame\
    				+	(unsigned	long)ERecParity\
    				+	(unsigned	long)ERecByteOverRun);
    	}
};

extern	ClassComs       	*Com[2];
#endif  //  __SMHCOM_HPP__
